package com.sun.java.swing.plaf.windows;

import javax.swing.text.Position;
import javax.swing.text.View;
import javax.swing.plaf.TextUI;
import java.awt.Rectangle;
import javax.swing.text.BadLocationException;
import javax.swing.text.JTextComponent;
import java.awt.Shape;
import java.awt.Graphics;
import java.awt.Color;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.Highlighter;
import javax.swing.plaf.UIResource;
import javax.swing.text.DefaultCaret;
import javax.swing.text.Caret;
import javax.swing.text.LayeredHighlighter;
import javax.swing.plaf.basic.BasicTextUI;

public abstract class WindowsTextUI extends BasicTextUI
{
    static LayeredHighlighter.LayerPainter WindowsPainter;
    
    public WindowsTextUI() {
    }
    
    @Override
    protected Caret createCaret() {
        return new WindowsTextUI.WindowsCaret();
    }
    
    static {
        WindowsTextUI.WindowsPainter = new WindowsTextUI.WindowsHighlightPainter(null);
    }
    
    static class WindowsCaret extends DefaultCaret implements UIResource
    {
        @Override
        protected Highlighter.HighlightPainter getSelectionPainter() {
            return WindowsTextUI.WindowsPainter;
        }
    }
    
    static class WindowsHighlightPainter extends DefaultHighlighter.DefaultHighlightPainter
    {
        WindowsHighlightPainter(Color c) {
            super(c);
        }
        
        @Override
        public void paint(Graphics graphics, int n, int n2, Shape shape, JTextComponent textComponent) {
            Rectangle bounds = shape.getBounds();
            try {
                TextUI ui = textComponent.getUI();
                Rectangle modelToView = ui.modelToView(textComponent, n);
                Rectangle modelToView2 = ui.modelToView(textComponent, n2);
                Color color = this.getColor();
                if (color == null) {
                    graphics.setColor(textComponent.getSelectionColor());
                }
                else {
                    graphics.setColor(color);
                }
                boolean n3 = false;
                boolean n4 = false;
                if (textComponent.isEditable()) {
                    int caretPosition = textComponent.getCaretPosition();
                    n3 = (n == caretPosition);
                    n4 = (n2 == caretPosition);
                }
                if (modelToView.y == modelToView2.y) {
                    Rectangle union = modelToView.union(modelToView2);
                    if (union.width > 0) {
                        if (n3 != 0) {
                            ++union.x;
                            --union.width;
                        }
                        else if (n4 != 0) {
                            --union.width;
                        }
                    }
                    graphics.fillRect(union.x, union.y, union.width, union.height);
                }
                else {
                    int n5 = bounds.x + bounds.width - modelToView.x;
                    if (n3 && n5 > 0) {
                        ++modelToView.x;
                        --n5;
                    }
                    graphics.fillRect(modelToView.x, modelToView.y, n5, modelToView.height);
                    if (modelToView.y + modelToView.height != modelToView2.y) {
                        graphics.fillRect(bounds.x, modelToView.y + modelToView.height, bounds.width, modelToView2.y - (modelToView.y + modelToView.height));
                    }
                    if (n4 && modelToView2.x > bounds.x) {
                        --modelToView2.x;
                    }
                    graphics.fillRect(bounds.x, modelToView2.y, modelToView2.x - bounds.x, modelToView2.height);
                }
            }
            catch (BadLocationException ex) {}
        }
        
        @Override
        public Shape paintLayer(Graphics graphics, int p6, int p7, Shape a, JTextComponent textComponent, View view) {
            Color color = this.getColor();
            if (color == null) {
                graphics.setColor(textComponent.getSelectionColor());
            }
            else {
                graphics.setColor(color);
            }
            boolean n = false;
            boolean n2 = false;
            if (textComponent.isEditable()) {
                int caretPosition = textComponent.getCaretPosition();
                n = (p6 == caretPosition);
                n2 = (p7 == caretPosition);
            }
            if (p6 == view.getStartOffset() && p7 == view.getEndOffset()) {
                Rectangle bounds;
                if ((a instanceof Rectangle)) {
                    bounds = (Rectangle)a;
                }
                else {
                    bounds = a.getBounds();
                }
                if (n && bounds.width > 0) {
                    graphics.fillRect(bounds.x + 1, bounds.y, bounds.width - 1, bounds.height);
                }
                else if (n2 && bounds.width > 0) {
                    graphics.fillRect(bounds.x, bounds.y, bounds.width - 1, bounds.height);
                }
                else {
                    graphics.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);
                }
                return bounds;
            }
            else {
                try {
                    Shape modelToView = view.modelToView(p6, Position.Bias.Forward, p7, Position.Bias.Backward, a);
                    Rectangle rectangle = (modelToView instanceof Rectangle) ? ((Rectangle)modelToView) : modelToView.getBounds();
                    if (n && rectangle.width > 0) {
                        graphics.fillRect(rectangle.x + 1, rectangle.y, rectangle.width - 1, rectangle.height);
                    }
                    else if (n2 && rectangle.width > 0) {
                        graphics.fillRect(rectangle.x, rectangle.y, rectangle.width - 1, rectangle.height);
                    }
                    else {
                        graphics.fillRect(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
                    }
                    return rectangle;
                }
                catch (BadLocationException ex) {
                    return null;
                }
            }
        }
    }
}
